﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;

namespace CellGrid
{
    public partial class Form1 : Form
    {
        int hoveredIndex = -1;
        public Form1()
        {
            InitializeComponent();

            #region [Data Soruce]
            //Creating and adding dataTable to the GGC 
            DataTable dt = new DataTable("MyTable");
            int nCols = 4;
            int nRows = 20;
            Random r = new Random(123345345);
            for (int i = 0; i < nCols; i++)
                dt.Columns.Add(new DataColumn(string.Format("Col{0}", i), typeof(int)));
            for (int i = 0; i < nRows; ++i)
            {
                DataRow dr = dt.NewRow();
                for (int j = 0; j < nCols; j++)
                    dr[j] = r.Next(10);
                dt.Rows.Add(dr);
            }
            this.gridGroupingControl1.DataSource = dt;

            dt = new DataTable("MyTable2");
            nRows = 20;
            for (int i = 0; i < nCols; i++)
                dt.Columns.Add(new DataColumn(string.Format("Colx{0}", i), typeof(int)));
            for (int i = 0; i < nRows; ++i)
            {
                DataRow dr = dt.NewRow();
                for (int j = 0; j < nCols; j++)
                    dr[j] = 100 + r.Next(10);
                dt.Rows.Add(dr);
            }
            this.gridGroupingControl2.DataSource = dt;
            #endregion

            this.gridGroupingControl1.AllowProportionalColumnSizing = true;
            this.gridGroupingControl2.AllowProportionalColumnSizing = true;

            //Setting the drop and selection properties.
            this.gridGroupingControl1.AllowDrop = true;
            this.gridGroupingControl1.TableOptions.ListBoxSelectionMode = SelectionMode.One;           
            this.gridGroupingControl1.TableControl.DefaultColWidth = 120;

            this.gridGroupingControl2.AllowDrop = true;
            this.gridGroupingControl2.TableOptions.ListBoxSelectionMode = SelectionMode.One;
            this.gridGroupingControl2.TableControl.DefaultColWidth = 120;
            this.gridGroupingControl2.ShowGroupDropArea = true;

            this.gridGroupingControl2.TableDescriptor.GroupedColumns.Add("Colx1");
            // Adding Summaries for the Details Table
            GridSummaryColumnDescriptor scd = new GridSummaryColumnDescriptor("Sum", SummaryType.DoubleAggregate, "Colx1", "{Sum:#}");
            GridSummaryRowDescriptor srd = new GridSummaryRowDescriptor("Sum", "$", scd);
            srd.Appearance.AnyCell.HorizontalAlignment = GridHorizontalAlignment.Right;
            srd.Appearance.AnyCell.BackColor = Color.FromArgb(255, 231, 162);
            this.gridGroupingControl2.GetTableDescriptor("MyTable2").SummaryRows.Add(srd);
            this.gridGroupingControl2.Table.TopLevelGroup.Groups[4].IsExpanded = true;

            // declaring the events.
            this.gridGroupingControl1.DragDrop += new DragEventHandler(grid_DragDrop);
            this.gridGroupingControl1.DragOver += new DragEventHandler(grid_DragOver);
            this.gridGroupingControl1.TableControlMouseDown += new GridTableControlMouseEventHandler(grid_TableControlMouseDown);
            //Invoke the events to change the color of row during the mouse hover.
            this.gridGroupingControl1.TableControlCellMouseHoverEnter += gridGroupingControl1_TableControlCellMouseHoverEnter;
            this.gridGroupingControl1.TableControlCellMouseHoverLeave += gridGroupingControl1_TableControlCellMouseHoverLeave;
            this.gridGroupingControl1.QueryCellStyleInfo += gridGroupingControl1_QueryCellStyleInfo;

            this.gridGroupingControl1.TableControlCurrentCellStartEditing += gridGroupingControl1_TableControlCurrentCellStartEditing;
            this.gridGroupingControl1.TableControlCurrentCellEditingComplete += gridGroupingControl1_TableControlCurrentCellEditingComplete;
            this.gridGroupingControl2.DragDrop += new DragEventHandler(grid_DragDrop);
            this.gridGroupingControl2.DragOver += new DragEventHandler(grid_DragOver);
            this.gridGroupingControl2.TableControlMouseDown += new GridTableControlMouseEventHandler(grid_TableControlMouseDown);

        }

        void gridGroupingControl1_TableControlCurrentCellEditingComplete(object sender, GridTableControlEventArgs e)
        {
            gridGroupingControl1.TableOptions.ListBoxSelectionMode = SelectionMode.MultiExtended;
            gridGroupingControl1.TableOptions.AllowSelection = GridSelectionFlags.None;     
        }

        void gridGroupingControl1_TableControlCurrentCellStartEditing(object sender, GridTableControlCancelEventArgs e)
        {            
            gridGroupingControl1.TableOptions.ListBoxSelectionMode = SelectionMode.None;           
            e.TableControl.CurrentCell.Activate(e.TableControl.CurrentCell.RowIndex, e.TableControl.CurrentCell.ColIndex);            
        }

      

        #region Event Handlers

        #region grid_DragDrop Event Handler

        void gridGroupingControl1_QueryCellStyleInfo(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableCellStyleInfoEventArgs e)
        {
            if (hoveredIndex == e.TableCellIdentity.RowIndex)
            {
                //Set the back color for the mouse hoverd row.
                e.Style.BackColor = Color.LightBlue;
            }
        }

        void gridGroupingControl1_TableControlCellMouseHoverLeave(object sender, GridTableControlCellMouseEventArgs e)
        {
            //Reset the mouse hoverd position while leave the row.
            hoveredIndex = -1;
        }

        void gridGroupingControl1_TableControlCellMouseHoverEnter(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlCellMouseEventArgs e)
        {
            //Set the hovered row index to change the backcolor.
            hoveredIndex = e.Inner.RowIndex;
            this.gridGroupingControl1.TableControl.Refresh();
        }
        

        private void grid_DragDrop(object sender, DragEventArgs e)
        {
            //getting the current grid.
            GridGroupingControl grid = sender as GridGroupingControl;
            //check if the data handled in this event is a SelectedRecordCollection.
            if (e.Data.GetDataPresent(typeof(SelectedRecordsCollection)))
            {
                SelectedRecordsCollection recs = e.Data.GetData(typeof(SelectedRecordsCollection)) as SelectedRecordsCollection;
                if (recs != null)
                {
                    GridTableControl tableControl = grid.TableControl;
                    // pt is used to get the coordinates where the data is dropped.
                    Point pt = tableControl.PointToClient(new Point(e.X, e.Y));
                    int row, col;
                    // using the point the corresponding row and column can be got.
                    if (tableControl.PointToRowCol(pt, out row, out col))
                    {
                        GridTableCellStyleInfo style = tableControl.Model[row, col];
                        // using the row and column the celltype can be found and checked if it is a RecordFieldCell.
                        if ((style.TableCellIdentity.TableCellType == GridTableCellType.AnyRecordFieldCell
                            || style.TableCellIdentity.TableCellType == GridTableCellType.AlternateRecordFieldCell))
                        {
                            //pasting the selected records in the target grid.
                            int target = grid.Table.FilteredRecords.IndexOf(style.TableCellIdentity.DisplayElement.ParentRecord);
                            foreach (SelectedRecord rec in recs)
                            {
                                DataRowView drv = rec.Record.GetData() as DataRowView;
                                if (target > -1 && target < grid.Table.FilteredRecords.Count)
                                {
                                    DataRowView targetDRV = grid.Table.FilteredRecords[target].GetData() as DataRowView; ;
                                    targetDRV.BeginEdit();
                                    for (col = 0; col < drv.Row.ItemArray.GetLength(0); col++)
                                    {
                                        if (col < targetDRV.Row.ItemArray.GetLength(0))
                                        {
                                            targetDRV[col] = drv[col];
                                        }
                                    }
                                    targetDRV.EndEdit();
                                    targetDRV.Row.AcceptChanges();
                                    target += 1;
                                }
                            }
                        }
                        //showing the effects during drag/drop.
                        e.Effect = DragDropEffects.Copy;// tell source grid not to delete records
                    }
                    else
                        e.Effect = DragDropEffects.None;
                }
            }
        }

        #endregion

        #region grid_DragOver Event Hadler

        /// <summary>
        /// Used to set the DragDrop effect
        /// </summary>
        private void grid_DragOver(object sender, DragEventArgs e)
        {
            // showing the dragdrop effects.
            if (e.Data.GetDataPresent(typeof(SelectedRecordsCollection)))
            {
                e.Effect = DragDropEffects.Copy;
            }
        }
        #endregion

        #region TableControlMouseDown Event Handler

        private void grid_TableControlMouseDown(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlMouseEventArgs e)
        {
            GridGroupingControl grid = sender as GridGroupingControl;
            if (grid != null)
            {
                //Clear all the selected records from the selection.
                grid.Table.SelectedRecords.Clear();
                GridTableControl tableControl = grid.TableControl;
                Point pt = new Point(e.Inner.X, e.Inner.Y);
                int row, col;
                if (tableControl.PointToRowCol(pt, out row, out col))
                {
                    GridTableCellStyleInfo style = tableControl.Model[row, col];
                    //checking if the cell in which the MouseDown event is a RecordRowHeaderCell.
                    //If yes, then begins the DragDrop operation.
                    if ((style.TableCellIdentity.TableCellType == GridTableCellType.RecordRowHeaderCell
                    || style.TableCellIdentity.TableCellType == GridTableCellType.AlternateRecordRowHeaderCell)
                    && style.TableCellIdentity.DisplayElement.ParentRecord != null)
                    {
                        //Add the Current record to the selection.
                        grid.Table.SelectedRecords.Add(style.TableCellIdentity.DisplayElement.GetRecord());
                        //Move the current cell to the clicked row col index.
                        grid.TableControl.CurrentCell.MoveTo(row, col);
                        //Refresh the grid to apply the changes.
                        grid.Refresh();
                        DragDropEffects ef = tableControl.DoDragDrop(new DataObject(grid.Table.SelectedRecords), DragDropEffects.All);
                        if (ef == DragDropEffects.Move)
                        {
                            //need to delete selection if you want to support this
                        }
                    }
                }
            }
        }
        #endregion

        #endregion
    }
}
